@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2020 Jeffrey Stedfast
+ *  Copyright (C) 2000-2022 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -174,8 +174,8 @@ write_message_to_screen (GMimeMessage *message)
 	GMimeStream *stream;
 	
 	/* create a new stream for writing to stdout */
-	stream = g_mime_stream_file_new (stdout);
-	g_mime_stream_file_set_owner ((GMimeStreamFile *) stream, FALSE);
+	stream = g_mime_stream_pipe_new (STDOUT_FILENO);
+	g_mime_stream_pipe_set_owner ((GMimeStreamPipe *) stream, FALSE);
 	
 	/* write the message to the stream */
 	g_mime_object_write_to_stream ((GMimeObject *) message, NULL, stream);
