@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2022 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -45,6 +45,8 @@ typedef struct _GMimeFilterStripClass GMimeFilterStripClass;
 struct _GMimeFilterStrip {
 	GMimeFilter parent_object;
 	
+	/* < private > */
+	void *lwsp;
 };
 
 struct _GMimeFilterStripClass {
